import java.awt.FlowLayout;
import java.io.*;
import java.net.*;
import java.util.*;

import javax.swing.SwingUtilities;

/*
 * Class: Server. 
 * 
 * Contains main function.
 */
public class Server
{
	private ArrayList<conBoard> cBoard;		//list of connected boards
	private int port;						//server port
	private ServerGUI sg;					//server GUI
	private ServerSocket serverSocket;
	private boolean state = false;			//connected?
	
	/*
	 * Server constructor
	 */
	public Server(int port)
	{
		this.port=port;
		this.sg=new ServerGUI(this);
		cBoard= new ArrayList<conBoard>();
		//cont=true;
	}
	
	/*
	 * Method: start
	 * 
	 * Listens for connections
	 * Creates a new conBoard for every new connection
	 */
	public void start() throws SocketException
	{ 
		this.sg.setState(true);
		this.state=true;
		Runnable rTask = new Runnable() {
			public void run() {
				try
				{
					serverSocket = new ServerSocket(port);
					while (true)	
					{
							Socket socket = serverSocket.accept();
							conBoard m = new conBoard(socket);
							cBoard.add(m);
							m.start();
					}
				}
				catch (IOException e)
				{
						String msg = e +"\n";
						System.out.println(msg);
				}
			}
		};
		Thread server = new Thread(rTask);
		server.start();
	}
	
	/*
	 * Method: stop
	 * 
	 * closes Server Socket 
	 */
	public void stop()
	{
		this.sg.setState(false);
		if (state)
		{
		try {
			serverSocket.close();
			for(conBoard c: cBoard)
			{
				c.socket.close();
				sg.removePanel(c.panel);
				cBoard.remove(c);
				c.connected=false;
				c.disposable=true;
			}
			
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		}
		this.state=false;
	}
	
	/*
	 * Method: setDisposable(String id)
	 * 
	 * conBoard id will be signaled as prepared to finish 
	 */
	public void setDisposable(String id)
	{
		for(conBoard c: cBoard)
		{
			if (c.id.equals(id))
			{	
				c.disposable=true;
				break;
			}
		}
	}
	
	/*
	 * main
	 * 
	 * Creates a Server object and runs it
	 */
	public static void main(String[] args)
	{
		//Server server = new Server(5432);
		//ServerGUI sg = new ServerGUI(server);
		//server.start();
		 SwingUtilities.invokeLater(new Runnable() {
	         @Override
	         public void run() {
	            new Server(5432); 
	         }
	      });
	}
	
	/*
	 * Inner class conBoard
	 * 
	 * One is created for every connected board.
	 * Manages the connection with the board & prints info at 
	 * the corresponding panel of the GUI.
	 */
	class conBoard extends Thread {
		private Socket socket;
		private String id;
		private boardPanel panel;
		private BufferedReader inFromClient;
		private DataOutputStream outToClient;
		private boolean connected=true;
		private boolean disposable=false;
		
		conBoard(Socket socket)
		{
			this.socket = socket;
	  		this.panel = new boardPanel(new FlowLayout(),sg);
	  		this.id="NNNNN";
	  		
	  		//we establish a 30 sec timeout on the socket
	  		try {
				this.socket.setSoTimeout(30000);
			} catch (SocketException e1) {
				// TODO Auto-generated catch block
				e1.printStackTrace();
			}
	  		
	  		try
	  		{
	  			inFromClient = new BufferedReader (new InputStreamReader(
					this.socket.getInputStream()));
	  			outToClient = new DataOutputStream(this.socket.getOutputStream());
	  		}
	  		catch (IOException e) {
	  			e.printStackTrace();
	  		}
		}
		
		/*
		 * Method: run()
		 * 
		 * reads and responds to board messages
		 * 
		 */
	  	public void run() 
	  	{
	  		String clientMessage;
	  		while (!disposable)
	  		{
	  			if(connected)
	  			{
	  				try 
	  				{	
	  					clientMessage = inFromClient.readLine();
	  					System.out.println(clientMessage);
	  				}
	  				catch (IOException e) 
	  				{	
	  					//if timeout expires, board is disconnected
	  					connected=false;
	  					clientMessage = "ERROR";
	  				}
	  				/* make the action according to board message */
	  				switch(clientMessage.substring(0,2)) {
	  				/*
	  				 * 10: fall
	  				 * 20: beat rate
	  				 * 30: board identification
	  				 * 40: battery level
	  				 */
	  				case "10":
	  					answer();
	  					this.panel.setAlarm(true);
	  					this.panel.setReset(true);
	  					break;
	  				case "20":
	  					answer();
	  					this.panel.setlFreqValue(clientMessage.substring(3,6));
	  					break;
	  				case "30":  
	  					answer();
	  					//mirar a veure si ja hi ha un board amb aquest id
	  					for(conBoard c: cBoard)
	  					{
	  						if (c.id.equals(clientMessage.substring(4,6)))
	  						{	
	  							sg.removePanel(c.panel);
	  							cBoard.remove(c);
	  							c.connected=false;
	  							c.disposable=true;
	  							break;
	  						}
	  					}
	  					this.id=clientMessage.substring(4,6);
  						this.panel.setId(this.id);
						sg.addPanel(this.panel);
						sg.revalidate();
	  					sg.repaint();	
	  					break;
	  				case "40": 
	  					answer();
	  					if (clientMessage.substring(4,6).equals("1"))
	  						this.panel.setBattery(true);
	  					else if (clientMessage.substring(4,6).equals("0"))
	  						this.panel.setBattery(false);
	  					break;
	  				}
	  			}
	  			else
	  			{
	  				this.panel.setlDiscon(true);
	  				this.panel.setRemove(true);
	  				
	  				while(true)
	  				{
	  					try {
	  						Thread.sleep(100);
	  					} catch (InterruptedException e) {
	  						// TODO Auto-generated catch block
	  						e.printStackTrace();
	  					}
	  					if (disposable)
	  					{
	  						sg.removePanel(this.panel);
	  						sg.revalidate();
		  					sg.repaint();	
	  						cBoard.remove(this);
	  						break;
	  					}
	  				}
	  			}
	  		}
	  	}
	  	
	  	/*
		 * Method: answer()
		 * 
		 * answer with an OK
		 * 
		 */
	  	private void answer(){
	  		try
			{
				outToClient.writeBytes("OK\n");
			}
			catch(IOException e)
			{
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
	  	}
	 }
}